package com.boman.system.controller;

import java.util.List;

import com.boman.domain.SysMenu;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.boman.domain.constant.Constants;
import com.boman.domain.constant.UserConstants;
import com.boman.common.core.utils.SecurityUtils;
import com.boman.common.core.utils.StringUtils;
import com.boman.common.core.web.controller.BaseController;
import com.boman.domain.dto.AjaxResult;
import com.boman.common.log.annotation.Log;
import com.boman.common.log.enums.BusinessType;
import com.boman.common.security.annotation.PreAuthorize;
import com.boman.system.service.ISysMenuService;

/**
 * 菜单信息
 * 
 * @author ruoyi
 */
@RestController
@RequestMapping("/menu")
public class SysMenuController extends BaseController
{
    @Autowired
    private ISysMenuService menuService;

    /**
     * 获取菜单列表
     */
    @PreAuthorize(hasPermi = "system:menu:list")
    @GetMapping("/list")
    public AjaxResult list(SysMenu menu)
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuList(menu, userId);
        return AjaxResult.success(menus);
    }

    /**
     * 获取所有菜单列表
     */
    @PostMapping("/listAll")
    public List<SysMenu> listAll(@RequestBody SysMenu menu)
    {
        return menuService.selectMenuListAll(menu);
    }

    /**
     * 获取某人的所有菜单
     */
    @GetMapping("/listMenusByUserId/{userId}")
    public List<SysMenu> list(@PathVariable Long userId) {
        return menuService.selectMenuList(new SysMenu(), userId);
    }

    /**
     * 根据菜单编号获取详细信息
     */
    @PreAuthorize(hasPermi = "system:menu:query")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable Long id)
    {
        return AjaxResult.success(menuService.selectMenuById(id));
    }

    /**
     * 获取菜单下拉树列表
     */
    @GetMapping("/treeselect")
    public AjaxResult treeselect(SysMenu menu)
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuList(menu, userId);
        return AjaxResult.success(menuService.buildMenuTreeSelect(menus));
    }

    /**
     * 加载对应角色菜单列表树, 不包含叶子结点 不包含叶子结点 不包含叶子结点 重要的事情说三遍
     */
    @GetMapping(value = "/treeMenuNotAddLeafNode/{roleId}")
    public AjaxResult treeMenuNotAddLeafNode(@PathVariable("roleId") Long roleId)
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuList(userId);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("checkedKeys", menuService.selectMenuListByRoleId(roleId));
        ajax.put("menus", menuService.buildMenuTreeSelectNotAddLeafNode(menus));
        return ajax;
    }

   /**
    * 功能描述: before: 根据roleId查找的叶子结点去匹配所对应的菜单树,
    * after: 不需要传roleId，查全部的，这样可以保证所有的角色都可以拿到所有的的菜单
    *
    * @param roleId roleId
    * @return com.boman.domain.dto.AjaxResult
    */
    @GetMapping(value = "/listTreeByRoleId/{roleId}")
    public AjaxResult listTreeByRoleId(@PathVariable("roleId") Long roleId) {
        return AjaxResult.success(menuService.listTreeByRoleId(null));
    }


    /**
     * 功能描述: 根据菜单id，找到此菜单以及子菜单下所有的叶子节点
     *
     * @param menuId menuId
     * @return com.boman.domain.dto.AjaxResult
     */
    /**
     * 功能描述: 根据菜单id，找到此菜单以及子菜单下所有的叶子节点
     *
     * @param menuId menuId
     * @return com.boman.domain.dto.AjaxResult
     */
    @GetMapping(value = "/allLeafNodeById/{menuId}")
    public AjaxResult allLeafNodeById(@PathVariable("menuId") Long menuId) {
        return AjaxResult.success(menuService.allLeafNodeById(menuId));
    }

    /**
     * 功能描述: 根据menuId和roleId，找到此菜单以及子菜单下所有的叶子节点, 同时匹配叶子结点对应的头部的规则
     *
     * @param menuId menuId
     * @param roleId roleIdmenu/listTreeByRoleId/1
     * @return com.boman.domain.dto.AjaxResult
     */
    @GetMapping(value = "/listMenus/roleId/{roleId}/menuId/{menuId}")
    public AjaxResult listMenus(@PathVariable("roleId") Long roleId, @PathVariable(value = "menuId", required = false) Long menuId) {
        return AjaxResult.success(menuService.listMenus(roleId, menuId));
    }

    /**
     * 新增菜单
     */
    @PreAuthorize(hasPermi = "system:menu:add")
    @Log(title = "菜单管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysMenu menu)
    {
        if (UserConstants.NOT_UNIQUE.equals(menuService.checkMenuNameUnique(menu)))
        {
            return AjaxResult.error("新增菜单'" + menu.getMenuName() + "'失败，菜单名称已存在");
        }
        else if (UserConstants.YES_FRAME.equals(menu.getIsFrame())
                && !StringUtils.startsWithAny(menu.getPath(), Constants.HTTP, Constants.HTTPS))
        {
            return AjaxResult.error("新增菜单'" + menu.getMenuName() + "'失败，地址必须以http(s)://开头");
        }
        menu.setCreateBy(SecurityUtils.getUsername());
        return menuService.insertMenu(menu);
    }

    /**
     * 修改菜单
     */
    @PreAuthorize(hasPermi = "system:menu:edit")
    @Log(title = "菜单管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysMenu menu)
    {
        if (UserConstants.NOT_UNIQUE.equals(menuService.checkMenuNameUnique(menu)))
        {
            return AjaxResult.error("修改菜单'" + menu.getMenuName() + "'失败，菜单名称已存在");
        }
        else if (UserConstants.YES_FRAME.equals(menu.getIsFrame())
                && !StringUtils.startsWithAny(menu.getPath(), Constants.HTTP, Constants.HTTPS))
        {
            return AjaxResult.error("修改菜单'" + menu.getMenuName() + "'失败，地址必须以http(s)://开头");
        }
        else if (menu.getId().equals(menu.getParentId()))
        {
            return AjaxResult.error("修改菜单'" + menu.getMenuName() + "'失败，上级菜单不能选择自己");
        }
        menu.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(menuService.updateMenu(menu));
    }

    /**
     * 删除菜单
     */
    @PreAuthorize(hasPermi = "system:menu:remove")
    @Log(title = "菜单管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{id}")
    public AjaxResult remove(@PathVariable("id") Long id)
    {
        if (menuService.hasChildById(id))
        {
            return AjaxResult.error("存在子菜单,不允许删除");
        }
        if (menuService.checkMenuExistRole(id))
        {
            return AjaxResult.error("菜单已分配,不允许删除");
        }
        return toAjax(menuService.deleteMenuById(id));
    }

    /**
     * 逻辑删除菜单
     */
    @PreAuthorize(hasPermi = "system:menu:remove")
    @Log(title = "菜单管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/remove/{id}")
    public AjaxResult delete(@PathVariable("id") Long id)
    {
        if (menuService.hasChildById(id))
        {
            return AjaxResult.error("存在子菜单,不允许删除");
        }
        if (menuService.checkMenuExistRole(id))
        {
            return AjaxResult.error("菜单已分配,不允许删除");
        }
        return toAjax(menuService.removeMenuById(id));
    }

    /**
     * 获取路由信息
     * 
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters()
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return AjaxResult.success(menuService.buildMenus(menus));
    }

    /**
     * 根据用户ID查询按钮
     *
     * @param userId 用户ID
     * @return 菜单列表
     */
    @GetMapping("listBtnByUserId/{userId}")
    public List<SysMenu> listBtnByUserId(@PathVariable("userId") Long userId) {
        return menuService.listBtnByUserId(userId);
    }

}