package com.boman.web.core.service.common;

import com.alibaba.fastjson.JSONObject;
import com.boman.common.redis.RedisKey;
import com.boman.common.redis.service.RedisService;
import com.boman.domain.GenTable;
import com.boman.domain.dto.*;
import com.boman.web.core.domain.RowResult;
import com.boman.web.core.service.TableServiceCmdService;
import com.boman.web.core.service.delete.IBaseDeleteService;
import com.boman.web.core.service.save.IBaseSaveService;
import com.boman.web.core.service.select.IBaseSelectService;
import com.boman.web.core.service.update.IBaseUpdateService;
import com.boman.web.core.utils.ColumnUtils;
import com.boman.web.core.utils.IdUtils;
import com.boman.web.core.utils.IdempotencyUtils;
import com.google.protobuf.DescriptorProtos;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;

import javax.annotation.Resource;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

import static com.boman.common.core.utils.obj.ObjectUtils.requireNonNull;

/**
 * @author shiqian
 * @date 2021年04月15日 16:31
 **/
@Service
public class CommonServiceImpl implements ICommonService {

    @Resource
    private IBaseSelectService selectService;
    @Resource
    private RedisService redisService;
    @Resource
    private TableServiceCmdService cmdService;
    @Resource
    private IBaseSaveService saveService;
    @Resource
    private IBaseDeleteService deleteService;
    @Resource
    private IBaseUpdateService updateService;

    /**
     * 功能描述: 根据表名和id查找
     *
     * @param tableName tableName
     * @param id        id
     * @return JSONObject
     */
    @Override
    public JSONObject getById(String tableName, Long id) {
        requireNonNull(tableName, "tableName is empty");
        requireNonNull(id, "id is empty");

        GenTable genTable = cmdService.getTableFromRedisByTableName(RedisKey.TABLE_INFO, tableName);
        String pkName = IdUtils.getPkName(genTable.getColumns());
        return selectService.selectById(tableName, pkName, id, "");
    }

    /**
     * 功能描述: 根据condition进行查找
     *
     * @param tableName tableName
     * @param condition condition
     * @return java.util.List<com.alibaba.fastjson.JSONObject>
     */
    @Override
    public List<JSONObject> getByMap(String tableName, JSONObject condition) {
        requireNonNull(tableName, "tableName is empty");
        return selectService.getByMap(tableName, condition);
    }

    /**
     * 功能描述: 根据tableName和map(属性名和属性值)进行查查
     *
     * @param tableName tableName
     * @param param     属性名和属性值
     * @return com.alibaba.fastjson.JSONObject
     */
    @Override
    public JSONObject getOneByMap(String tableName, JSONObject param) {
        requireNonNull(tableName, "表名为空");
        return selectService.getOneByMap(tableName, param);
    }

    @Override
    public Long getMaxId(String tableName, String pkName) {
        return IdUtils.getMaxId(tableName, pkName);
    }

    /**
     * 功能描述: 根据条件count
     *
     * @param dto dto
     * @return java.lang.Long
     */
    @Override
    public int count(FormDataDto dto) {
        JSONObject condition = dto.getFixedData();
        String tableName = dto.getTable();
        requireNonNull(tableName, "tableName is empty");
        GenTable genTable = cmdService.getTableFromRedisByTableName(RedisKey.TABLE_INFO, tableName);
        JSONObject packCondition = ColumnUtils.packColCondition(genTable.getColumns(), condition);
        return selectService.countByCondition(tableName, packCondition, condition);
    }

    /**
     * 功能描述: 获取最新的一条数据 create desc
     *
     * @param tableName tableName
     * @return jcom.alibaba.fastjson.JSONObject
     */
    @Override
    public JSONObject getNewest(String tableName) {
        requireNonNull(tableName, "tableName is empty");
        return selectService.getNewest(tableName);
    }

    /**
     * 功能描述: 批量插入数据 默认插入的是多对多的关联表，因此未处理create_by等字段
     *
     * @param dto dto
     * @return int
     */
    @Override
    public int insertRelList(InsertListDto dto) {
        String tableName = requireNonNull(dto.getTableName(), "tableName is empty");
        List<JSONObject> dataList = requireNonNull(dto.getDataList(), "dataList is empty");

        return saveService.insertRelList(tableName, dataList);
    }

    /**
     * 功能描述: 通用修改数据
     *
     * @param dto dto
     * @return int
     */
    @Override
    public List<RowResult> insertList(InsertListDto dto) {
        String tableName = requireNonNull(dto.getTableName(), "tableName is empty");
        List<JSONObject> dataList = requireNonNull(dto.getDataList(), "dataList is empty");

        return saveService.insertList(tableName, dataList);
    }

    /**
     * 功能描述: 通用删除数据
     *
     * @param dto dto
     * @return int
     */
    @Override
    public int delete(DeleteDto dto) {
        String tableName = requireNonNull(dto.getTableName(), "tableName is empty");
        JSONObject condition = requireNonNull(dto.getCondition(), "condition is empty");

        return deleteService.delete(tableName, condition);
    }

    /**
     * 功能描述: 通用修改数据
     *
     * @param dto dto
     * @return int
     */
    @Override
    public int update(UpdateDto dto) {
        String tableName = requireNonNull(dto.getTableName(), "tableName is empty");
        JSONObject condition = requireNonNull(dto.getCondition(), "condition is empty, 条件为空不允许调用修改接口，防止误操作");
        JSONObject commitData = requireNonNull(dto.getCommitData(), "commitData is empty");

        return updateService.update(tableName, commitData, condition);
    }

    @Override
    public AjaxResult genSubmitToken(String tableName, String username) {
        String idempotencyToken = IdempotencyUtils.packKey(tableName, username);
        redisService.setCacheObject(idempotencyToken, idempotencyToken, 1L, TimeUnit.HOURS);
        return AjaxResult.success(idempotencyToken);
    }
}
